###############################################################################
#
# Copyright (c) 2017 Ruslan Spivak, Maxim Muzafarov, Murad Byashimov
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

__author__ = 'Maxim Muzafarov <m.muzafarov@gmail.com>'

import os

from setuptools import setup, find_packages


classifiers = """\
Intended Audience :: Developers
License :: OSI Approved :: MIT License
Programming Language :: Python
Topic :: Software Development :: Testing
Operating System :: Unix
"""


def read(*rel_names):
    return open(os.path.join(os.path.dirname(__file__), *rel_names)).read()


setup(
    name='cynic3',
    version='3.0',
    url='https://github.com/m-messiah/cynic3',
    license='MIT',
    description='Test harness to make your system under test cynical',
    author='Maxim Muzafarov',
    author_email='m.muzafarov@gmail.com',
    packages=find_packages(),
    zip_safe=False,
    install_requires=['logsna'],
    entry_points="""\
    [console_scripts]
    cynic = cynic.server:main
    """,
    classifiers=[x for x in classifiers.split('\n') if x],
    long_description=read('README.rst') + '\n\n' + read('CHANGES.rst'),
)
